@@ -24,6 +24,10 @@ struct ipset_arg;
 typedef int (*ipset_parsefn)(struct ipset_session *s,
 			     enum ipset_opt opt, const char *str);
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern int ipset_parse_ether(struct ipset_session *session,
 			     enum ipset_opt opt, const char *str);
 extern int ipset_parse_port(struct ipset_session *session,
@@ -82,6 +86,8 @@ extern int ipset_parse_uint64(struct ipset_session *session,
 			      enum ipset_opt opt, const char *str);
 extern int ipset_parse_uint32(struct ipset_session *session,
 			      enum ipset_opt opt, const char *str);
+extern int ipset_parse_uint16(struct ipset_session *session,
+			      enum ipset_opt opt, const char *str);
 extern int ipset_parse_uint8(struct ipset_session *session,
 			     enum ipset_opt opt, const char *str);
 extern int ipset_parse_netmask(struct ipset_session *session,
@@ -94,6 +100,10 @@ extern int ipset_parse_iface(struct ipset_session *session,
 			     enum ipset_opt opt, const char *str);
 extern int ipset_parse_comment(struct ipset_session *session,
 			     enum ipset_opt opt, const char *str);
+extern int ipset_parse_skbmark(struct ipset_session *session,
+			      enum ipset_opt opt, const char *str);
+extern int ipset_parse_skbprio(struct ipset_session *session,
+				enum ipset_opt opt, const char *str);
 extern int ipset_parse_output(struct ipset_session *session,
 			      int opt, const char *str);
 extern int ipset_parse_ignored(struct ipset_session *session,
@@ -110,4 +120,8 @@ extern int ipset_parse_iptimeout(struct ipset_session *session,
 extern int ipset_parse_name_compat(struct ipset_session *session,
 				   enum ipset_opt opt, const char *str);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* LIBIPSET_PARSE_H */
