@@ -13,6 +13,10 @@ typedef int (*ipset_printfn)(char *buf, unsigned int len,
 			     const struct ipset_data *data,
 			     enum ipset_opt opt, uint8_t env);
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern int ipset_print_ether(char *buf, unsigned int len,
 			     const struct ipset_data *data,
 			     enum ipset_opt opt, uint8_t env);
@@ -77,4 +81,8 @@ extern int ipset_print_data(char *buf, unsigned int len,
 			    const struct ipset_data *data,
 			    enum ipset_opt opt, uint8_t env);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* LIBIPSET_PRINT_H */
