@@ -180,24 +180,27 @@ int
 ipset_parse_ether(struct ipset_session *session,
 		  enum ipset_opt opt, const char *str)
 {
-	unsigned int i = 0;
+	size_t len, p = 0, i = 0;
 	unsigned char ether[ETH_ALEN];
 
 	assert(session);
 	assert(opt == IPSET_OPT_ETHER);
 	assert(str);
 
-	if (strlen(str) != ETH_ALEN * 3 - 1)
+	len = strlen(str);
+
+	if (len > ETH_ALEN * 3 - 1)
 		goto error;
 
 	for (i = 0; i < ETH_ALEN; i++) {
 		long number;
 		char *end;
 
-		number = strtol(str + i * 3, &end, 16);
+		number = strtol(str + p, &end, 16);
+		p = end - str + 1;
 
-		if (end == str + i * 3 + 2 &&
-		    (*end == ':' || *end == '\0') &&
+		if (((*end == ':' && i < ETH_ALEN - 1) ||
+		     (*end == '\0' && i == ETH_ALEN - 1)) &&
 		    number >= 0 && number <= 255)
 			ether[i] = number;
 		else
@@ -1606,6 +1609,23 @@ ipset_parse_uint32(struct ipset_session *session,
 	return err;
 }
 
+int
+ipset_parse_uint16(struct ipset_session *session,
+		   enum ipset_opt opt, const char *str)
+{
+	uint16_t value;
+	int err;
+
+	assert(session);
+	assert(str);
+
+	err = string_to_u16(session, str, &value);
+	if (err == 0)
+		return ipset_session_data_set(session, opt, &value);
+
+	return err;
+}
+
 /**
  * ipset_parse_uint8 - parse string as an unsigned short integer
  * @session: session structure
@@ -1803,6 +1823,55 @@ int ipset_parse_comment(struct ipset_session *session,
 	return ipset_data_set(data, opt, str);
 }
 
+int
+ipset_parse_skbmark(struct ipset_session *session,
+		    enum ipset_opt opt, const char *str)
+{
+	struct ipset_data *data;
+	uint64_t result = 0;
+	unsigned long mark, mask;
+	int ret = 0;
+
+	assert(session);
+	assert(opt == IPSET_OPT_SKBMARK);
+	assert(str);
+
+	data = ipset_session_data(session);
+	ret = sscanf(str, "0x%lx/0x%lx", &mark, &mask);
+	if (ret != 2) {
+		mask = 0xffffffff;
+		ret = sscanf(str, "0x%lx", &mark);
+		if (ret != 1)
+			return syntax_err("Invalid skbmark format, "
+					  "it should be: "
+					  " MARK/MASK or MARK (see manpage)");
+	}
+	result = ((uint64_t)(mark) << 32) | (mask & 0xffffffff);
+	return ipset_data_set(data, IPSET_OPT_SKBMARK, &result);
+}
+
+int
+ipset_parse_skbprio(struct ipset_session *session,
+		    enum ipset_opt opt, const char *str)
+{
+	struct ipset_data *data;
+	unsigned maj, min;
+	uint32_t major;
+	int err;
+
+	assert(session);
+	assert(opt == IPSET_OPT_SKBPRIO);
+	assert(str);
+
+	data = ipset_session_data(session);
+	err = sscanf(str, "%x:%x", &maj, &min);
+	if (err != 2)
+		return syntax_err("Invalid skbprio format, it should be:"\
+				  "MAJOR:MINOR (see manpage)");
+	major = ((uint32_t)maj << 16) | (min & 0xffff);
+	return ipset_data_set(data, IPSET_OPT_SKBPRIO, &major);
+}
+
 /**
  * ipset_parse_output - parse output format name
  * @session: session structure
@@ -1976,8 +2045,10 @@ ipset_parse_elem(struct ipset_session *session,
 		D("parse elem part two: %s", a);
 		parse_elem(session, type, IPSET_DIM_TWO, a);
 	}
-	if (type->dimension > IPSET_DIM_TWO && b != NULL)
+	if (type->dimension > IPSET_DIM_TWO && b != NULL) {
+		D("parse elem part three: %s", b);
 		parse_elem(session, type, IPSET_DIM_THREE, b);
+	}
 
 	goto out;
 
